#!/bin/sh

[ -r /etc/opt/drweb.com/ldwrap.conf ] && . /etc/opt/drweb.com/ldwrap.conf
[ -r /usr/local/etc/opt/drweb.com/ldwrap.conf ] && . /usr/local/etc/opt/drweb.com/ldwrap.conf

die() {
    retval=$1
    shift
    echo "Fatal error: $*" >&2
    exit $retval
}

warning() {
    echo "Warning: $*" >&2
}

debug() {
    if [ -n "$DEBUG" ] ; then
        echo "Debug: $*" >&2
    fi
}

get_platform() {
    uname -s | tr "[:upper:]" "[:lower:]"
}

get_lib_path() {
    binary="$1"
    lib="$2"
    $LDD $LDD_RECURSIVE "$binary" 2>&1 | grep "$lib" | head -n1 | while read lib dummy path ; do echo $path ; done | cut -d' ' -f1
}

binary_grep() {
    pattern="$1"
    binary="$2"
    retval=0
    [ -r "$binary" ] || return 1
    if grep --help | grep 'GNU' >/dev/null 2>&1 ; then
        grep -q "$pattern" "$binary" >/dev/null 2>&1
        retval=$?
    else
        #sun generic grep cannot parse binary
        cat "$binary" | sed "s/$pattern/@@@OK@@@/" | grep '@@@OK@@@' >/dev/null 2>&1
        retval=$?
    fi
    return $retval
}

append_to_library_path() {
    debug "Append \"$1\" to LD_LIBRARY_PATH"
    if [ ! -z "$LD_LIBRARY_PATH" ]; then
        ldBasePath="$LD_LIBRARY_PATH"
    fi
    if [ ! -z "$LD_32_LIBRARY_PATH" ]; then
        ld32BasePath="$LD_32_LIBRARY_PATH"
    fi
    LD_32_LIBRARY_PATH="$ld32BasePath:$LIBPATH/$1"
    export LD_32_LIBRARY_PATH
    LD_LIBRARY_PATH="$ldBasePath:${LIBPATH}64/$1:$LIBPATH/$1"
    export LD_LIBRARY_PATH
}

check_glibc_version() {
    glibcMajor="`$LDD --version | awk '/^ldd/ { split($NF,v,/\./); print v[1]; }'`"
    glibcMinor="`$LDD --version | awk '/^ldd/ { split($NF,v,/\./); print v[2]; }'`"
    glibcBugfix="`$LDD --version | awk '/^ldd/ { split($NF,v,/\./); print v[3]; }'`"
    if [ "$glibcMinor" = "" ]; then
       warning "Could not determine system's glibc version."
       glibcMinor=0
       return
    fi
    if [ "$glibcBugfix" = "" ]; then
       glibcBugfix=0
    fi
    minVersion=`echo $MIN_GLIBC_VERSION | awk -F. '{print $1*10000+$2*100+$3}'`
    glibcVersion=`expr $glibcMajor \* 10000 + $glibcMinor \* 100 + $glibcBugfix`

    if [ $glibcVersion -lt $minVersion ]; then
        USE_OWN_GLIBC="TRUE"
        kernel=`uname -r | awk -F. '{print $1*10000+$2*100+$3}'`
        if test $kernel -ge 20600 ; then
            KERNEL="2.6"
        elif test $kernel -ge 20400 ; then
            KERNEL="2.4"
        else
            die 125 "You need kernel 2.4 or later."
        fi
    fi
}

run() {
    libs=""
    loader=""
    if [ "$PLATFORM" = "linux" ] ; then
        check_glibc_version
        if [ -n "$USE_OWN_GLIBC" ] ; then
            loader="$LIBPATH/ldwrap/$KERNEL/ld-linux.so.2"
            append_to_library_path "ldwrap/$KERNEL"
            append_to_library_path "ldwrap/gcc"
            append_to_library_path "ldwrap/libz.so.1"
        fi
        lddOutput="`LANGUAGE=C LANG=C $LDD $LDD_RECURSIVE "$BINARY" 2>&1`"
        if echo "$lddOutput" | grep gzopen64 >/dev/null 2>&1 ; then
            append_to_library_path "ldwrap/libz.so.1"
        fi
    elif [ "$PLATFORM" = "freebsd" ] ; then
        #rpath doesn't work here
        append_to_library_path ""
    fi
    if binary_grep "libstdc++\.so\.6" "$BINARY" ; then
        libpath="`get_lib_path "$BINARY" "libstdc++.so.6" 2>/dev/null`"
        if binary_grep "CXXABI_$MIN_CXXABI" "$libpath" ; then
            true
        else
            append_to_library_path "ldwrap/gcc"
        fi
    fi
    #last check Qt
    lddOutput="`LANGUAGE=C LANG=C $LDD $LDD_RECURSIVE "$BINARY" 2>&1`"
    echo "$lddOutput" | grep "Qt" | grep "not found" >/dev/null 2>&1 || echo "$lddOutput" | grep "undefined symbol" >/dev/null 2>&1
    need_qt=$?
    if [ $need_qt -eq 0 ] ; then
        append_to_library_path "qt4/lib"
    fi
    debug "Finaly LD_LIBRARY_PATH is \"$LD_LIBRARY_PATH\""
    if [ -n "$loader" ] ; then
        $LOADER "$BINARY" "$@"
        retval=$?
    else
        exec "$BINARY" "$@"
        retval=$?
    fi
    return $retval
}

LDD=`which ldd`
[ -x "$LDD" ] || die 125 "ldd not found. Fix PATH and try again."

PLATFORM=`get_platform`

case "$PLATFORM" in
    *bsd*)
        LDD_RECURSIVE="${LDD_RECURSIVE:-"-a"}"
        PREFIX="${PREFIX:-"/usr/local/drweb.com"}"
        ;;
    *)
        LDD_RECURSIVE="${LDD_RECURSIVE:-"-r"}"
        PREFIX="${PREFIX:-"/opt/drweb.com"}"
        ;;
esac

LIBPATH="$PREFIX/lib"
BINARY="$PREFIX/bin/`basename "$0.real"`"

[ -x "$BINARY" ] || die 104 "$BINARY not found or not executable"

MIN_GLIBC_VERSION=2.3.4
MIN_CXXABI=1.3

run "$@"
exit $?
